@@ -1,8 +1,8 @@
 #include "device_msg.h"
 
+#include <stdlib.h>
 #include <string.h>
 
-#include "config.h"
 #include "util/buffer_util.h"
 #include "util/log.h"
 
@@ -21,7 +21,7 @@ device_msg_deserialize(const unsigned char *buf, size_t len,
             if (clipboard_len > len - 5) {
                 return 0; // not available
             }
-            char *text = SDL_malloc(clipboard_len + 1);
+            char *text = malloc(clipboard_len + 1);
             if (!text) {
                 LOGW("Could not allocate text for clipboard");
                 return -1;
@@ -43,6 +43,6 @@ device_msg_deserialize(const unsigned char *buf, size_t len,
 void
 device_msg_destroy(struct device_msg *msg) {
     if (msg->type == DEVICE_MSG_TYPE_CLIPBOARD) {
-        SDL_free(msg->clipboard.text);
+        free(msg->clipboard.text);
     }
 }
