@@ -1,34 +1,37 @@
 #ifndef RECORDER_H
 #define RECORDER_H
 
+#include "common.h"
+
 #include <stdbool.h>
 #include <libavformat/avformat.h>
-#include <SDL2/SDL_mutex.h>
-#include <SDL2/SDL_thread.h>
 
-#include "config.h"
-#include "common.h"
+#include "coords.h"
 #include "scrcpy.h"
+#include "trait/packet_sink.h"
 #include "util/queue.h"
+#include "util/thread.h"
 
 struct record_packet {
-    AVPacket packet;
+    AVPacket *packet;
     struct record_packet *next;
 };
 
 struct recorder_queue QUEUE(struct record_packet);
 
 struct recorder {
+    struct sc_packet_sink packet_sink; // packet sink trait
+
     char *filename;
     enum sc_record_format format;
     AVFormatContext *ctx;
     struct size declared_frame_size;
     bool header_written;
 
-    SDL_Thread *thread;
-    SDL_mutex *mutex;
-    SDL_cond *queue_cond;
-    bool stopped; // set on recorder_stop() by the stream reader
+    sc_thread thread;
+    sc_mutex mutex;
+    sc_cond queue_cond;
+    bool stopped; // set on recorder_close()
     bool failed; // set on packet write failure
     struct recorder_queue queue;
 
@@ -46,22 +49,4 @@ recorder_init(struct recorder *recorder, const char *filename,
 void
 recorder_destroy(struct recorder *recorder);
 
-bool
-recorder_open(struct recorder *recorder, const AVCodec *input_codec);
-
-void
-recorder_close(struct recorder *recorder);
-
-bool
-recorder_start(struct recorder *recorder);
-
-void
-recorder_stop(struct recorder *recorder);
-
-void
-recorder_join(struct recorder *recorder);
-
-bool
-recorder_push(struct recorder *recorder, const AVPacket *packet);
-
 #endif
